import * as React from 'react';
import Header from './header';
import Row, { Column } from './row';
import { combineClasses } from '../lib/util';
/**
 * Base Table class
 *
 * Since React doesn't allow type parameters to be set in jsx, i'd suggest the creation of a custom class which extends this base class and includes a type parameter
 * @export
 * @class Table
 * @extends {React.Component<ITableProps<T>, Partial<ITableState>>}
 * @template T
 */
export class Table extends React.Component {
    constructor() {
        super();
        this.state = {};
        this.setSort = this.setSort.bind(this);
    }
    render() {
        const tGridSize = this.props.gridSize || 12;
        const tClasses = [
            'table',
            'container',
            this.props.className
        ];
        const tHeaderColumnClasses = [
            this.props.headerColumnClassName,
            this.props.addClassesToHeader !== false ? this.props.columnClassName : ''
        ];
        const tHeaderClasses = [
            this.props.headerClassName,
            this.props.addClassesToHeader !== false ? this.props.rowClassName : ''
        ];
        let tData = this.props.data;
        if (this.state.sortField) {
            tData = tData.sort((a, b) => {
                const tAField = a[this.state.sortField];
                const tBField = b[this.state.sortField];
                const tSortedFieldConfig = this.props.fieldConfig[this.state.sortField];
                const tSortFactor = this.state.sortDir === 'asc' ? 1 : -1;
                if (tSortedFieldConfig.sortFunction) {
                    return tSortedFieldConfig.sortFunction(tAField, tBField, tSortFactor);
                }
                let tSort;
                if (typeof (tAField) === 'string') {
                    tSort = tAField.localeCompare(tBField);
                }
                else if (typeof (tAField) === 'number') {
                    tSort = tAField - tBField;
                }
                else {
                    tSort = 1;
                }
                return tSortFactor * tSort;
            });
        }
        return (React.createElement("div", { className: tClasses.join(' ') },
            React.createElement(Header, { sortField: this.state.sortField, sortDir: this.state.sortDir, sortable: this.props.sortable, key: 'header', data: this.props.fieldConfig, className: combineClasses(tHeaderClasses), columnClassName: combineClasses(tHeaderColumnClasses), filteredClassName: this.props.filteredColumnClassName, onColumClick: this.setSort }),
            tData.map((entry, index) => {
                let tKey = this.props.data[index][this.props.primaryKey] || index;
                return React.createElement(Row, { key: tKey, sortField: this.state.sortField, rowKey: tKey, data: entry, config: this.props.fieldConfig, className: this.props.rowClassName, columnClasses: this.props.columnClassName, filteredColumnClasses: this.props.filteredColumnClassName, onColumnClick: (rowKey, columnKey) => {
                        if (this.props.onRowSelected) {
                            this.props.onRowSelected(rowKey);
                        }
                        if (this.props.onCellClick) {
                            this.props.onCellClick(rowKey, columnKey);
                        }
                    } });
            })));
    }
    setSort(key) {
        if (this.props.fieldConfig[key].sortable === false) {
            return;
        }
        if (key !== this.state.sortField) {
            return this.setState({
                sortField: key,
                sortDir: this.state.sortDir === undefined ? 'asc' : this.state.sortDir
            });
        }
        this.setState({
            sortDir: this.state.sortDir === 'asc' ? 'desc' : 'asc'
        });
    }
}
export default Table;
export { Row, Header, Column };
//# sourceMappingURL=index.js.map